<template>
  <div 
    v-if="OneStep.visible" 
    :style="menuStyle" 
    class="context_menu menu_fadein"
    :class="{ vertical: topMenuDirection === 'y', horizontal: topMenuDirection === 'x' }"
    @touchstart="handleTouchStart"
    @touchmove="handleScroll"
  >
    <template v-for="(item, index) in menuData" :key="index">
      <div
        class="menu_item"
        :class="{ highlighted: index === highlightedIndex }"
        :style="getItemStyle(item)"
        @mouseover="handleMouseOver(index, item)"
        @touchmove="handleTouchMove($event, index, item)"
      >
        {{ item.name }}
        <!-- 二级菜单 -->
        <div v-if="(index === highlightedIndex || activeSubmenu === index) && item.children" 
             :style="getSubmenuStyle(item.direction, item)" 
             class="submenu"
             :class="{ vertical: item.direction === 'y', horizontal: item.direction === 'x' }">
          <template v-for="(subitem, subindex) in item.children" :key="subindex">
            <div
              class="submenu_item"
              :class="{ highlighted: subindex === subhighlightedIndex }"
              :style="getItemStyle(subitem)"
              @mouseover="handleSubMouseOver(index, subindex, subitem)"
              @touchmove="handleSubTouchMove($event, index, subindex, subitem)"
            >
              {{ subitem.name }}
              <!-- 三级菜单 -->
              <div v-if="(subindex === subhighlightedIndex || activeThirdMenu === `${index}-${subindex}`) && subitem.children" 
                   :style="getThirdmenuStyle(subitem.direction, subitem)" 
                   class="submenu"
                   :class="{ vertical: subitem.direction === 'y', horizontal: subitem.direction === 'x' }">
                <template v-for="(thirditem, thirdindex) in subitem.children" :key="thirdindex">
                  <div
                    class="submenu_item"
                    :class="{ highlighted: thirdindex === thirdhighlightedIndex }"
                    :style="getItemStyle(thirditem)"
                    @mouseover="handleThirdMouseOver(index, subindex, thirdindex, thirditem)"
                    @touchmove="handleThirdTouchMove($event, index, subindex, thirdindex, thirditem)"
                  >
                    {{ thirditem.name }}
                  </div>
                </template>
              </div>
            </div>
          </template>
        </div>
      </div>
    </template>
  </div>
</template>

<script setup>
import { ref, inject, provide, onMounted, onBeforeUnmount } from 'vue';

const OneStep = inject('OneStep');

const menuStyle = ref({});
const menuData = ref([]);
const topMenuDirection = ref('x');
const highlightedIndex = ref(-1);
const subhighlightedIndex = ref(-1);
const thirdhighlightedIndex = ref(-1);
const activeSubmenu = ref(-1);
const activeThirdMenu = ref(null);

const touchStartX = ref(0);
const touchStartY = ref(0);
const isHorizontalScroll = ref(false);

OneStep.visible = false;

OneStep.show = (event, data, direction = 'x') => {
  const x = event.clientX || event.touches[0].clientX;
  const y = event.clientY || event.touches[0].clientY;
  const target = event.target;
  console.log('坐标:', x, y, '按下的DOM对象:', target);

  menuData.value = data;
  topMenuDirection.value = direction;

  menuStyle.value = {
    top: `${y}px`,
    left: `${x + 10}px`,
    position: 'fixed'
  };

  OneStep.visible = true;

  document.addEventListener('mouseup', handleMouseUp);
  document.addEventListener('touchend', handleTouchEnd);
};

const handleMouseUp = (event) => {
  const element = document.elementFromPoint(event.clientX, event.clientY);
  triggerFunForElement(element);
  OneStep.hide(event);
};

const handleTouchEnd = (event) => {
  if (event.changedTouches && event.changedTouches.length > 0) {
    const touch = event.changedTouches[0];
    const element = document.elementFromPoint(touch.clientX, touch.clientY);
    triggerFunForElement(element);
  }
  OneStep.hide(event);
};

const triggerFunForElement = (element) => {
  if (!element) return;

  let currentElement = element;
  let item;
  let level;

  // 从当前元素向上查找，直到找到菜单项或达到顶层
  while (currentElement && !item) {
    if (currentElement.classList.contains('menu_item')) {
      const index = Array.from(currentElement.parentElement.children).indexOf(currentElement);
      item = menuData.value[index];
      level = 1;
    } else if (currentElement.classList.contains('submenu_item')) {
      const subindex = Array.from(currentElement.parentElement.children).indexOf(currentElement);
      const parentMenuItem = currentElement.closest('.menu_item');
      if (parentMenuItem) {
        const parentIndex = Array.from(parentMenuItem.parentElement.children).indexOf(parentMenuItem);
        if (currentElement.closest('.submenu > .submenu_item > .submenu')) {
          // 三级菜单
          item = menuData.value[parentIndex].children[activeSubmenu.value].children[subindex];
          level = 3;
        } else {
          // 二级菜单
          item = menuData.value[parentIndex].children[subindex];
          level = 2;
        }
      }
    }
    currentElement = currentElement.parentElement;
  }

  if (item && item.fun && typeof item.fun === 'function') {
    console.log(`触发 ${level} 级菜单项的 fun 函数: ${item.name}`);
    item.fun();
  }
};

OneStep.hide = (event) => {
  const x = event.clientX || (event.changedTouches && event.changedTouches[0].clientX);
  const y = event.clientY || (event.changedTouches && event.changedTouches[0].clientY);
  const target = event.target;
  console.log('坐标:', x, y, '松开的DOM对象:', target);
  OneStep.visible = false;
  highlightedIndex.value = -1;
  subhighlightedIndex.value = -1;
  thirdhighlightedIndex.value = -1;
  activeSubmenu.value = -1;
  activeThirdMenu.value = null;
  document.removeEventListener('mouseup', handleMouseUp);
  document.removeEventListener('touchend', handleTouchEnd);
};

const handleMouseOver = (index, item) => {
  highlightedIndex.value = index;
  if (item.children) {
    activeSubmenu.value = index;
  } else {
    activeSubmenu.value = -1;
  }
  activeThirdMenu.value = null;
};

const handleSubMouseOver = (parentIndex, subindex, subitem) => {
  subhighlightedIndex.value = subindex;
  if (subitem.children) {
    activeThirdMenu.value = `${parentIndex}-${subindex}`;
  } else {
    activeThirdMenu.value = null;
  }
};

const handleThirdMouseOver = (parentIndex, subIndex, thirdindex, thirditem) => {
  thirdhighlightedIndex.value = thirdindex;
};

const handleTouchMove = (event, index, item) => {
  const touch = event.touches[0];
  const element = document.elementFromPoint(touch.clientX, touch.clientY);
  const currentIndex = Array.from(document.querySelectorAll('.menu_item')).indexOf(element);
  if (currentIndex !== -1) {
    handleMouseOver(currentIndex, menuData.value[currentIndex]);
  }
};

const handleSubTouchMove = (event, parentIndex, subindex, subitem) => {
  const touch = event.touches[0];
  const element = document.elementFromPoint(touch.clientX, touch.clientY);
  const currentSubIndex = Array.from(document.querySelectorAll('.submenu_item')).indexOf(element);
  if (currentSubIndex !== -1) {
    handleSubMouseOver(parentIndex, currentSubIndex, menuData.value[parentIndex].children[currentSubIndex]);
  }
};

const handleThirdTouchMove = (event, parentIndex, subIndex, thirdindex, thirditem) => {
  const touch = event.touches[0];
  const element = document.elementFromPoint(touch.clientX, touch.clientY);
  const currentThirdIndex = Array.from(document.querySelectorAll('.submenu_item')).indexOf(element);
  if (currentThirdIndex !== -1) {
    handleThirdMouseOver(parentIndex, subIndex, currentThirdIndex, menuData.value[parentIndex].children[subIndex].children[currentThirdIndex]);
  }
};

const getSubmenuStyle = (direction, item) => {
  return {
    top: direction === 'y' ? '100%' : '0px',
    left: direction === 'x' ? '100%' : '0px',
  };
};

const getThirdmenuStyle = (direction, subitem) => {
  return {
    top: direction === 'y' ? '100%' : '0px',
    left: direction === 'x' ? '100%' : '0px',
  };
};

const getItemStyle = (item) => {
  return {
    width: item.width ? `${item.width}px` : 'auto',
    height: item.height ? `${item.height}px` : 'auto',
    minWidth: item.minWidth ? `${item.minWidth}px` : 'auto',
    minHeight: item.minHeight ? `${item.minHeight}px` : 'auto',
  };
};

const handleTouchStart = (event) => {
  touchStartX.value = event.touches[0].clientX;
  touchStartY.value = event.touches[0].clientY;
  isHorizontalScroll.value = false;
};

const handleScroll = (event) => {
  if (!isHorizontalScroll.value) {
    const touchEndX = event.touches[0].clientX;
    const touchEndY = event.touches[0].clientY;
    const deltaX = Math.abs(touchEndX - touchStartX.value);
    const deltaY = Math.abs(touchEndY - touchStartY.value);

    if (deltaX > deltaY) {
      isHorizontalScroll.value = true;
      event.preventDefault();
    }
  } else {
    event.preventDefault();
  }
};

onMounted(() => {
  provide('OneStep', OneStep);
  document.addEventListener('touchstart', handleTouchStart, { passive: false });
  document.addEventListener('touchmove', handleScroll, { passive: false });
});

onBeforeUnmount(() => {
  document.removeEventListener('mouseup', handleMouseUp);
  document.removeEventListener('touchend', handleTouchEnd);
  document.removeEventListener('touchstart', handleTouchStart);
  document.removeEventListener('touchmove', handleScroll);
});
</script>

<style scoped>
.context_menu {
  background: white;
  border: 1px solid #000;
  height: auto;
  width: auto;
  box-shadow: 0 2px 10px rgba(0, 0, 0, 0.1);
  transform: translateY(-50%);
  z-index: 9999999999;
}

.context_menu.horizontal {
  display: flex;
  flex-direction: row;
}

.context_menu.vertical {
  display: flex;
  flex-direction: column;
}

.menu_item {
  padding: 5px;
  position: relative;
}

.menu_item.highlighted {
  background-color: gray;
}

.submenu {
  position: absolute;
  background: white;
  border: 1px solid #000;
  box-shadow: 0 2px 10px rgba(0, 0, 0, 0.1);
  z-index: 10;
}

.submenu.horizontal {
  display: flex;
  flex-direction: row;
}

.submenu.vertical {
  display: flex;
  flex-direction: column;
}

.submenu_item {
  padding: 5px;
  position: relative;
}

.submenu_item.highlighted {
  background-color: gray;
}






@keyframes menu_fadein {
  0% {
    opacity: 0;
  }
  30% {
    opacity: 0;
  }
  100% {
    opacity: 1;
  }
}

.menu_fadein {
  animation: menu_fadein 0.6s forwards;
}

</style>